import cliutils, docopt, os, sequtils, tempfile

import strawbosspkg/configuration
import strawbosspkg/core
import strawbosspkg/server

let SB_VER = "0.2.0"

proc logProcOutput*(outMsg, errMsg: TaintedString, cmd: string) =
  let prefix = if cmd != nil: cmd else: ""
  if outMsg != nil: echo prefix & "(stdout): " & outMsg
  if errMsg != nil: echo prefix & "(stderr): " & errMsg

when isMainModule:

  let doc = """
Usage:
  strawboss serve [options]
  strawboss run <project> <step> [options]
  strawboss hashpwd <pwd>
  
Options
  
  -c --config-file <cfgFile>  Use this config file instead of the default
                              (strawboss.config.json).

  -f --force-rebuild          Force a build step to re-run even we have cached
                              results from building that step before for this
                              version of the project.

  -r --reference <ref>        Build the project at this commit reference.

  -w --workspace <workspace>  Use the given directory as the build workspace.

"""

  let args = docopt(doc, version = "strawboss v" & SB_VER)

  let cfgFile = if args["--config-file"]: $args["--config-file"]
                else: "strawboss.config.json"

  var cfg = loadStrawBossConfig(cfgFile)
  cfg.pathToExe = paramStr(0)
  if not existsDir(cfg.artifactsRepo):
    echo "Artifacts repo (" & cfg.artifactsRepo & ") does not exist. Creating..."
    createDir(cfg.artifactsRepo)

  cfg.artifactsRepo = expandFilename(cfg.artifactsRepo)


  if args["run"]:

    let wkspDir = if args["--workspace"]: $args["--workspace"] else: mkdtemp()

    try:
      let req = RunRequest(
        projectName: $args["<project>"],
        stepName: $args["<step>"],
        buildRef: if args["--reference"]: $args["--reference"] else: nil,
        forceRebuild: args["--force-rebuild"],
        workspaceDir: wkspDir)

      let status = core.initiateRun(cfg, req, logProcOutput)
      if status.state == "failed": raiseEx status.details
      echo "strawboss: build passed."
    except:
      echo "strawboss: build FAILED: " & getCurrentExceptionMsg() & "."
      quit(QuitFailure)
    finally:
      if existsDir(wkspDir): removeDir(wkspDir)

  elif args["serve"]: server.start(cfg)

  elif args["hashpwd"]:
    echo $cfg.pwdCost
    let pwd = server.hashPwd($args["<pwd>"], cfg.pwdCost)
    echo pwd
    echo pwd[0..28]

